#!/bin/bash

##########################################################################################
### scripts de vérification des pré-requis avant instanciation d'un eCDL
##########################################################################################
script=$0
scriptPath="/prerequis"

# hack parseDico
if [ -s '/root/zephir.eol' ]; then
        mv /root/zephir.eol /etc/eole/config.eol
else
        if [ -s '/etc/eole/config.eol' ]; then
                echo "$script:Le fichier /root/zephir.eol a déjà été déplacé vers /etc/eole/config.eol"
                [ -e '/root/config.eol' ] && rm -f /root/config.eol
        else
                if [ ! -s '/root/config.eol' ]; then
                        echo "$script:Il faut récupérer ou générer le fichier de configuration .eol"
                        exit 1
                else
                        echo "$script:Le fichier /root/config.eol est déjà généré, le script est ré-exécuté"
                        rm -f /root/config.eol
                fi
        fi
fi

# utilisation des librairies Eole
cd /root
if [ -x /usr/share/eole/FonctionsEoleNg ]
then
. /usr/share/eole/FonctionsEoleNg
else
        echo "fichier FonctionsEoleNG non trouvé"
        ecritLog $script "ERR" "EOLE" "fichier FonctionsEoleNg non trouvé"
        exit 1
fi

# utilisation de variables du dictionnaire
. /usr/bin/ParseDico

##################################################################################################
##
## fonctions
##

log ()
{
        logger -t EQDOMINST -s -f $LOG "$1" >> $LOG 2>&1
}

injecte_eol()
{
        ecdl_sid=$1
        eq_dom=$2
        echo "ecdl_smb_sid='${ecdl_sid}'" > $fichierInjection
        echo "ecdl_smb_DOM_exist='${eq_dom}'" >> $fichierInjection
        $scriptInjection

}

test_reseau()
{

#initialisation des variables
#IP du DNS national à pinger
p_ipdnsnat="10.167.160.3"

#Quelques couleurs (rouge test pas glop - vert test réussi)
NORMAL="\\033[0;39m"
ROUGE="\\033[1;31m"
VERT="\\033[1;32m"

#fin initialisation des variables

if [ ! -s /prerequis/exxl_test_reseau ];then
    echo -e "$p_nom: $ROUGE Le fichier de tests réseau /usr/bin/exxl_test_reseau n'existe pas ou est vide=> arrêt de l'installation$NORMAL"
    exit 1
fi

adresse_serveur=$(ifconfig eth0 | grep "inet adr" | cut -d: -f2 | awk '{print $1}')

if [ "$adresse_serveur" != "$adresse_ip_eth0" ];then
    echo -e "$p_nom: $ROUGE l'adresse IP du serveur: $adresse_serveur\nne correspond pas à l'adresse IP définie\npour la configuration récupérée du zephir pour ce contrôleur: $adresse_ip_eth0 $NORMAL"
    exit 1
fi

#Début des tests réseau
. /prerequis/exxl_test_reseau
#Fin des tests réseau

}


function par_feu_ldap_port
{
#$1 nom du serveur ldap et $2 port utilisé
#option de nc: w timeout de 5s, pas d'E/S de données (scanne)
#nc -w 5 -z -v ${eq_serv_ldap} 389 >> $LOG 2>&1
if `nc -w 5 -z -v $1 $2 >> $LOG 2>&1`;then
    echo "$p_nom:Le pare-feu de la centrale est ouvert de ip:${eq_serv_ip} -> $1 port $2" >> $LOG 2>&1
    #Mettre en # la ligne suivante
    #echo "Le pare-feu de la centrale est ouvert de ip:${eq_serv_ip} -> $1 port $2"
else
    echo "$p_nom:ATTENTION:Le pare-feu de la Centrale est fermé pour $1 (port $2) à partir de:${eq_serv_ip}" >> $LOG 2>&1
    echo -e "$ROUGE$p_nom: ATTENTION:Le pare-feu de la Centrale est fermé pour $1 (port $2) à partir de:${eq_serv_ip}.$NORMAL"

    PARFEU="KO"
fi
}

test_parefeu()
{

PARFEU="OK" #Par-feu de la Centrale ouvert
# LDAP_MAITRE="ldapma.csac.melanie2.i2" #... :-) LDAP Maître
LDAP_MAITRE="$adresse_ip_ldap_maitre" #... :-) LDAP Maître

eq_serv_ldap=$adresse_ip_ldap_distant
eq_serv_ip=$adresse_ip_eth0

# récupération de l'@IP du ldap Maître

ADR_MAITRE="10.167.128.9"
NOM_LONG_MAITRE="ldapma-01.csac.melanie2.i2"
NOM_COURT_MAITRE="ldapma"

for ip_host in $adresse_ip_hosts
  do
        if [ "$nom_long_hosts" = "$adresse_ip_ldap_maitre" ]; then
                ADR_MAITRE=$ip_host
                NOM_LONG_MAITRE=$nom_long_hosts
                NOM_COURT_MAITRE=$nom_court_hosts
        fi
  done


# Ajout du serveur ldap maître
[ $(cat /etc/hosts|grep "^$ADR_MAITRE $NOM_LONG_MAITRE $NOM_COURT_MAITRE"|wc -l) -eq 0 ] && echo "$ADR_MAITRE $NOM_LONG_MAITRE $NOM_COURT_MAITRE" >> /etc/hosts

#cd /usr/share/${eq_modele}

echo "$p_nom:Test de l'ouverture du pare-feu de la Centrale (timeout de 5s si pb)"  >> $LOG 2>&1
echo "$p_nom:Test de l'ouverture du pare-feu de la Centrale (timeout de 5s si pb)"

PARFEU=$PARFEU #Par-feu de la Centrale ouvert

#Test l'accès au ldaps smb et master sur le $ecdl_ldap_port
#par_feu_ldap_port ${eq_serv_ldap} 389
par_feu_ldap_port ${eq_serv_ldap} $ecdl_ldap_port
#par_feu_ldap_port $LDAP_MAITRE 389
par_feu_ldap_port $LDAP_MAITRE $ecdl_ldap_port

if [ $PARFEU = "OK" ] ; then
    echo "$p_nom:Le pare-feu de la centrale est ouvert de ${eq_serv_ip} vers l'annuaire LDAP" >> $LOG 2>&1
    echo "$p_nom:Le pare-feu de la centrale est ouvert de ${eq_serv_ip} vers l'annuaire LDAP"
else
   echo "$p_nom: Arrêt de l'instanciation."
   exit 1 #sortie du script avec erreur
fi

}

test_lit_ldap()
{

#Ajout du certificat pour pouvoir accéder à l'annuaire en ldaps
        [ $(cat /etc/ldap/ldap.conf|grep -i CA2008.pem|wc -l) -eq 0 ] && echo "TLS_CACERT /etc/certs/CA2008.pem" >> /etc/ldap/ldap.conf

LIMITETEMPS=30        # Nbre de secondes d'attente avant validation
echo "$p_nom:Test si le domaine existe dans l'annuaire LDAP: $adresse_ip_ldap_distant:$ecdl_ldap_port">> $LOG 2>&1

echo
echo -e " ___ Test de connexion ldaps: Attente de$VERT $LIMITETEMPS secondes$NORMAL ____ "
echo -e "|                          |"
echo -e "|$VERT ATTENTION:en l'absence de réponse le script considére$NORMAL |"
echo -e "|$VERT que votre nom de domaine est correct.$NORMAL            |"
echo -e "| Si celui-ci n'apparait pas: il y a un problème de    |"
echo "| connexion ldaps, faites ^C ou répondre non (n),validez  |"
echo "| puis vérifiez les préalables à l'installation.  |"
echo "|_______________________________________________________|"
# eq_serv_ip=`hostname -i`
eq_serv_ip=$adresse_ip_eth0

DOM=`/prerequis/dom_ip_ac.pl $eq_serv_ip`

if [ "$DOM" == "" ] ;then
    echo "Il n'y a pas de nom de domaine retourné"
    echo "=> Arrêt de l'installation"
    exit 1
fi

if [ "$(echo $DOM|tr [:upper:] [:lower:])" != "$ecdl_smb_workgroup" ] ;then
    echo -e "l'adresse IP du serveur correspond à un contrôleur du domaine $VERT $(echo $DOM|tr [:upper:] [:lower:]) $NORMAL, ce qui ne correspond pas au domaine saisi sur le zephir pour ce serveur $ROUGE $ecdl_smb_workgroup $NORMAL"
    echo "=> Arrêt de l'installation"
    exit 1
fi


echo
echo -e "Domaine obtenu dans ldap : $VERT $DOM $NORMAL"
echo "Si votre domaine s'affiche ci-dessus, on continue, sinon répondre (n)on avant $LIMITETEMPS secondes"

echo "Domaine OK ? o/n"

read -t $LIMITETEMPS reponse

echo

#Si 30 s écoulées on teste si domaine contient une valeur
if [ -z "$reponse" ] ;then # Est nul ?
        echo "$p_nom:$LIMITETEMPS secondes écoulées, on teste si un domaine est retourné ou non." >> $LOG 2>&1

        echo "$p_nom:$LIMITETEMPS secondes écoulées, on teste si un domaine est retourné ou non."
    if [ -n "$DOM" ] ;then # n'est pas nul
        echo "$p_nom:Le domaine $DOM existe, on sort du script et on continue." >> $LOG 2>&1
        echo "$p_nom:Le domaine $DOM existe, on continue."
    else
        echo "$p_nom:Aucun domaine trouvé=> sortie avec erreur (arrêt des scripts)" >> $LOG 2>&1
        echo -e "$p_nom: $ROUGE Aucun domaine trouvé $NORMAL => sortie avec erreur (arrêt des scripts)"
        exit 1 #Aucun domaine trouvé=> sortie avec erreur
    fi
else
    if [ $reponse = o ] ;then
        echo "$p_nom:Réponse oui=> le domaine est visible, on continue" >> $LOG 2>&1
        echo "$p_nom:Réponse oui=> le domaine est visible, on continue"
    else
        echo "$p_nom:Réponse non=> sortie avec erreur (arrêt des scripts)" >> $LOG 2>&1
        echo "$p_nom:Réponse non=> sortie avec erreur (arrêt des scripts)"
        exit 1 #Réponse non=> sortie avec erreur
    fi
fi

}


test_ecrit_ldap()
{

eq_pass=$ecdl_adm_pass_default

SIG=$ecdl_sig
DOM=$eq_serv_domsamba



[ ! -d "/usr/share/ecdl/" ] && mkdir /usr/share/ecdl
cd /usr/share/ecdl

cat > /tmp/test_ecrit_tmp.ldif  << EOF
###################################
# Test d'écriture dans l'annuaire LDAP
###################################
## test_ecrit
dn: ou=test_ecrit,ou=$DOM$meddtl_const_virgule$ldap_base_dn
objectClass: organizationalUnit
objectClass: top
ou: test_ecrit
EOF

cat > /tmp/test_ecrit_del_tmp.ldif  << EOF
ou=test_ecrit,ou=$DOM$meddtl_const_virgule$ldap_base_dn
EOF

echo "$p_nom: essai d'écriture dans l'annuaire ldap avec le mot de passe par défaut puis avec le nouveau mot de passe">> $LOG 2>&1
echo "$p_nom: essai d'écriture dans l'annuaire ldap avec le mot de passe par défaut puis avec le nouveau mot de passe"


#Test avec le mot de passe par défaut lors de la 1er boucle
MotDePasse=$eq_pass
#puis avec le nouveau mot de passe dans la 2eme boucle
#Resultat de l'écriture ldaps: OK ça fonctionne (resultat_ecrit_defaut avec le mot de passe par défaut amedee)
resultat_ecrit="OK"
resultat_ecrit_defaut="OK"

for (( i=1; i <= 2 ; i++ ))
do
    #echo "debut de boucle i vaut $i"

   echo "$p_nom:Suppression de ou=test_ecrit au cas où elle existe (ne pas en tenir compte)">> $LOG 2>&1
   ldapdelete -c -v -x -w $MotDePasse -D uid=SambaAdm.$DOM,ou=$DOM$meddtl_const_virgule$ldap_base_dn -H $ecdl_ldap_protocole://$adresse_ip_ldap_maitre:$ecdl_ldap_port/ -f /tmp/test_ecrit_del_tmp.ldif>> $LOG 2>&1

   #ajout de ou=test_ecrit
   echo "$p_nom:Ajout de ou=test_ecrit">> $LOG 2>&1
   ldapmodify -a -v -x -w $MotDePasse -D uid=SambaAdm.$DOM,ou=$DOM$meddtl_const_virgule$ldap_base_dn -H $ecdl_ldap_protocole://$adresse_ip_ldap_maitre:$ecdl_ldap_port/ -f /tmp/test_ecrit_tmp.ldif>> $LOG 2>&1

   if [ $? != 0 ] ;then
       echo "$p_nom: l'écriture de ou=test_ecrit dans $DOM a échoué avec un des mdp: les ACL ne sont pas appliquées:arrêt des scripts">> $LOG 2>&1
       resultat_ecrit="KO"
   else
        #Suppression de ou=test_ecrit
        ldapdelete -c -v -x -w $MotDePasse -D uid=SambaAdm.$DOM,ou=$DOM$meddtl_const_virgule$ldap_base_dn -H $ecdl_ldap_protocole://$adresse_ip_ldap_maitre:$ecdl_ldap_port/ -f /tmp/test_ecrit_del_tmp.ldif>> $LOG 2>&1

        if [ $? != 0 ] ;then
            echo "$p_nom: la suppression de ou=test_ecrit dans $DOM a échoué avec un des mdp: les ACL ne sont pas appliquées:arrêt des scripts">> $LOG 2>&1
            resultat_ecrit="KO"
        else
            echo "$p_nom: l'écriture de ou=test_ecrit dans $DOM et sa suppression ont réussi avec un des mdp: les ACL sont bien positionnées">> $LOG 2>&1
            resultat_ecrit="OK"
        fi
    fi
    if  [ $MotDePasse = $eq_pass ]; then
        resultat_ecrit_defaut=$resultat_ecrit
        resultat_ecrit="OK"
    fi

    #Debut de récup du nouveau mot de passe ---------------------
    echo $SIG> cle_equidom
    wget --no-proxy http://${eq_site}/diff_cle/recup_cle_enc.php -O cle.enc -o /dev/null
    wget --no-proxy http://${eq_site}/diff_cle/recup_pass_enc.php -O pass.enc -o /dev/null

    #echo "La cle de crypt. est `cat cle.enc` le pass crypt `cat pass.enc` "
    if [ -r cle.enc ]; then
        openssl enc -d -aes256 -in cle.enc -out .cle -kfile cle_equidom
        openssl enc -d -aes256 -in pass.enc -out .pass -kfile .cle
    fi

    echo "$p_nom: on reéssaye avec le nouveau mot de passe">> $LOG 2>&1
    #Si .pass existe et n'est pas vide
    if [ -s .pass ]; then
        MotDePasse=`cat .pass`
    else
        MotDePasse='xyz'
    fi

    #debug a supprimer
    #echo "Le mot de passe récupéré de buttle est `cat .pass`"
    #echo "le fichier /etc/sysconfig/ contient `cat /etc/sysconfig/.pass`"

    #Suppression des fichiers temporaires
    rm cle.enc pass.enc cle_equidom .cle .pass
    #Fin de récup du nouveau mot de passe ---------------------
    #echo "fin de boucle i vaut $i"
done

    #un peu de ménage
    rm -f /tmp/test_ecrit_tmp.ldif>> $LOG 2>&1
    rm -f /tmp/test_ecrit_del_tmp.ldif>> $LOG 2>&1

}



gestion_sid()
{
        echo "$p_nom: Recherche SID du domaine dans LDAP"
        ldap_eq_sambaSID=`ldapsearch -x  -s one -H $ecdl_ldap_protocole://$adresse_ip_ldap_distant:$ecdl_ldap_port/ -b ou=$ecdl_smb_workgroup$meddtl_const_virgule$ldap_base_dn objectClass=sambaDomain sambaSID | grep -v "^#" | grep "^sambaSID:" | tail -1 | cut -d ' ' -f 2`
        echo "$p_nom: Fin de recherche (${ldap_eq_sambaSID})"

        if [ "${ldap_eq_sambaSID}" == "" ] ; then

                echo "$p_nom:le SID du domaine ${eq_serv_domsamba}, n'existe pas encore dans ldap."

                if [ "$ecdl_smb_sid" == "" ] ; then

                                echo "$p_nom:Celui-ci sera généré"
                                eq_sambaSID=`net getlocalsid |cut -d ':' -f 2|tr -d ' '`

                else

                                echo "$p_nom:le SID utilisé sera celui saisi via Zephir : ${ecdl_smb_sid}"
                                eq_sambaSID=$ecdl_smb_sid
                fi

                eq_DOM_exist=0

        else

                echo "$p_nom:On utilise le SID inscrit dans LDAP"
                eq_sambaSID=$ldap_eq_sambaSID
                eq_DOM_exist=1
        fi


        echo "$p_nom:Le SID utilisé est ${eq_sambaSID}"
}


gestion_mdp()
{

echo "$p_nom version $p_version du $p_date : Initialisation des mots de passe "
echo "$p_nom version $p_version du $p_date : Initialisation des mots de passe " >> $LOG 2>&1

[ ! -d "/etc/sysconfig/" ] && mkdir /etc/sysconfig
[ ! -d "/usr/share/ecdl/" ] && mkdir /usr/share/ecdl


cd /usr/share/${eq_modele}

if [ ! -r /etc/sysconfig/.pass ] ;  then

	# echo "Creation .pass"
	echo "${eq_pass:amedee}" > /etc/sysconfig/.pass

fi

echo $SIG> cle_equidom

wget --no-proxy http://${eq_site}/diff_cle/recup_cle_enc.php -O cle.enc -o /dev/null
wget --no-proxy http://${eq_site}/diff_cle/recup_pass_enc.php -O pass.enc -o /dev/null

#echo "La cle de crypt. est `cat cle.enc` le pass crypt `cat pass.enc` "

if [ -r cle.enc ]

then
	openssl enc -d -aes256 -in cle.enc -out .cle -kfile cle_equidom
	openssl enc -d -aes256 -in pass.enc -out .pass -kfile .cle

fi

if [ -r /etc/sysconfig/.pass ]

then

	cp -f /etc/sysconfig/.pass /etc/sysconfig/.pass_old

fi

#echo "la cle apres decryp est `cat .pass`"

mv -f .pass .cle /etc/sysconfig/
rm cle_equidom cle.enc pass.enc

# echo "eq_pass=`cat /etc/sysconfig/.pass`" > param.tmp
echo "`cat /etc/sysconfig/.pass`" > param.tmp
eq_pass_adm=`cat /etc/sysconfig/.pass`


# c'est comme si la modif avait réussi
MOD=0

if [ `cat /etc/sysconfig/.pass` == `cat /etc/sysconfig/.pass_old` ]

then

	echo "$p_nom: ancien et nouveau mot de passe identique pour SambaAdm.$eq_serv_domsamba, rien à faire"
	echo "$p_nom: ancien et nouveau mot de passe identique pour SambaAdm.$eq_serv_domsamba, rien à faire">> $LOG 2>&1

elif [ "$eq_DOM_exist" == "1" ]

then

	#le domaine existe, c'est un BDC, on ne force pas le mdp ldap
	echo  "$p_nom: le domaine existe, mot de passe ldap déjà changé"

else

	echo "$p_nom: mot de passe en cours de changement"

	DOM=$eq_serv_domsamba


	echo "$p_nom: essai de modification du mot de passe Ldap"


    ldappasswd -x -H $ecdl_ldap_protocole://$adresse_ip_ldap_maitre:$ecdl_ldap_port/ -D "uid=SambaAdm.$DOM,ou=$DOM$meddtl_const_virgule$ldap_base_dn" -w $(cat /etc/sysconfig/.pass_old) -s $(cat param.tmp) 2>> $LOG > /dev/null


	MOD=$?

	echo "$p_nom: code retour ldappasswd $MOD"
	echo "$p_nom: code retour ldappasswd $MOD"  >> $LOG 2>&1

	rm param.tmp

fi

##ici on change les mots de passe samba et ldap tool

if [ "$MOD" == "0" ]

then

      echo "$p_nom: mise a jour dans samba"

      smbpasswd -w `cat /etc/sysconfig/.pass` >> $LOG 2>&1

      # mise à jour du fichier smbldap_bind.conf via le template smbldap_bind.conf.ecdl

      #echo "$p_nom: mise à jour dans /etc/smbldap-tools/smbldap_bind"
      #echo "eq_pass=`cat /etc/sysconfig/.pass`" > param.tmp

      # repl_fic.sh modeles/smbldap_bind.conf.std smbldap_bind.conf.tmp param.tmp >> $LOG 2>&1
      # repl_fic.sh smbldap_bind.conf.tmp /etc/smbldap-tools/smbldap_bind.conf >> $LOG 2>&1

      # rm smbldap_bind.conf.tmp param.tmp

else

      echo "$p_nom: *** ERREUR Impossible de modifier ldap ***"

      cp /etc/sysconfig/.pass /etc/sysconfig/.pass_tmp
      rm /etc/sysconfig/.pass
      mv /etc/sysconfig/.pass_old /etc/sysconfig/.pass

fi

#Protection du fichier pass
chmod 600 /etc/sysconfig/.pass*
chown root /etc/sysconfig/.pass*

}

##################################################################################################

#############################################
# variables historiques
#############################################
p_version="2.0.0"
p_date="2012.05.09"
p_nom=${0##*/}
p_projet=utilitaires

eq_rep_log=/var/log/ecdl
eq_serv_domsamba=$(echo $ecdl_smb_workgroup|tr [:lower:] [:upper:])
eq_site=$serveur_metier
eq_modele=$ecdl_type_serv
eq_DOM_exist=$ecdl_smb_DOM_exist
SIG=$ecdl_sig
eq_pass=$ecdl_adm_pass_default

LOG=${eq_rep_log}/install_${p_projet}.log

mkdir -p ${eq_rep_log}
touch $LOG


#############################################
# ajout de variables

fichierInjection=/tmp/injecte.ini
scriptInjection=/prerequis/injection
#############################################


#############################################
# déroulement des pré-requis
#############################################
m="DEBUT_:PGM=$p_nom:VERSION=$p_version:DATE=$p_date"
log "$m"

# tests réseaux pare-feu AC et accès lecture/écriture annuaire
test_reseau
test_parefeu
test_lit_ldap
test_ecrit_ldap

# recupération ou création du sid.
gestion_sid

gestion_mdp


# On injecte les trois valeurs dans le fichier.eol pour l'instanciation
injecte_eol $eq_sambaSID $eq_DOM_exist

# archivage du fichier /etc/eole/config avant traitement
mv /etc/eole/config.eol  /etc/eole/config.eol.old

# copie du fichier résultat vers le répertoire /etc/eole afin que les deux fichiers soient à niveau
cp -f /root/config.eol /etc/eole/config.eol
#protection du mdp
chmod 600 /root/config.eol /etc/eole/config.eol /etc/eole/config.eol.old

# ménage
rm /tmp/injecte.ini


m="FIN___:PGM=$p_nom:VERSION=$p_version:DATE=$p_date"
log "$m"

exit 0




